@@ -5,6 +5,7 @@
 
 #include "global.h"
 #include "slang.h"
+#include <unistd.h>
 
 #define TIMEOUT 2 /* 2/10 of a second */
 
@@ -12,6 +13,8 @@
 int getch (void) {
   int ch;
 
+  if (tcgetpgrp(0) == -1) return 033; /* Detect if the terminal went away. */
+  
   while (0 == SLang_input_pending(1000))
     continue;
 
